/*
  This code implements a computation of the sun's and the moon's positions 
  as required for xmoontool.c, utilizing libnova.
  
  Permission  to  use, copy, modify, and distribute this software and its
  documentation  for  any  purpose  and  without  fee  is  hereby  granted,
  without any conditions or restrictions. This software is provided "as is" 
  without expressed or implied warranty.

  (C) 2006  Frank W. Josellis
*/

/* nova.c */

#include <math.h>

#include <libnova/ln_types.h>
#include <libnova/earth.h>
#include <libnova/solar.h>
#include <libnova/lunar.h>


#ifndef PI
#ifdef  M_PI 
#define PI M_PI
#else
#define PI 3.14159265358979323846
#endif
#endif

#define AU        149597870.691  /* Astronomical Unit in km               */  
#define SANGLE    1919.26        /* Sun's angular size in arc seconds
				    at distance 1 AU                      */


#define deg2rad(x) ((x) * PI/180.0)
#define fixangle(x) ((x) - 360.0 * (floor((x)/360.0)))       /* Fix angle */

#define cosd(x) cos(deg2rad((x)))


/*-------------------------------------------------------------------------*/
/* Function `phase2'                                                       */
/* Replacement for xmoontool's phase()                                     */
/*-------------------------------------------------------------------------*/
double phase2(jd, phase, mage, r_m, mangle, r_s, sangle)
  double  jd;
  double  *phase;		      /* Illuminated fraction */
  double  *mage;		      /* Age of moon in days */
  double  *r_m;		              /* Moon's distance in km */
  double  *mangle;		      /* Moon's angular diameter in degrees */
  double  *r_s;		              /* Sun's distance in km */
  double  *sangle;                    /* Sun's angular diameter in degrees */
{
  double cos_elong;
  struct ln_lnlat_posn psun, pmoon;

  /* Get the sun's distance in AU. */
  *r_s = ln_get_earth_solar_dist(jd);

  /* Get the sun's ecliptic coordinates. */
  ln_get_solar_ecl_coords(jd, &psun);

  /* The sun's angular diameter in degrees. */
  *sangle = (SANGLE / *r_s) / 3600;
  
  /* Convert the sun's distance to km. */
  *r_s *= AU;

  /* Get the moon's distance in km. */
  *r_m = ln_get_lunar_earth_dist(jd);
  
  /* Get the moon's ecliptic coordinates. */
  ln_get_lunar_ecl_coords(jd, &pmoon, 0.0001);
  
  /* Get the moon's angular diameter in degrees. */
  *mangle = (2 * ln_get_lunar_sdiam(jd)) / 3600;

  /* Get the moon's illuminated fraction. */
  cos_elong = cosd(psun.lng - pmoon.lng) * cosd(pmoon.lat);
  *phase = (1 - cos_elong) / 2;

  /* mage is no longer used. */
  *mage = 0;

  return(fixangle(pmoon.lng - psun.lng) / 360.0);
}
