/*
  This code implements a computation of the sun's and the moon's positions 
  as required for xmoontool.c, based on the description in Paul Schlyter's 
  tutorial (cf. http://stjarnhimlen.se/comp/tutorial.html).
  
  Permission  to  use, copy, modify, and distribute this software and its
  documentation  for  any  purpose  and  without  fee  is  hereby  granted,
  without any conditions or restrictions. This software is provided "as is" 
  without expressed or implied warranty.

  (C) 2006  Frank W. Josellis
*/

/* schlyter.c */

#include <stdlib.h>
#include <math.h>

#ifndef PI
#ifdef  M_PI 
#define PI M_PI
#else
#define PI 3.14159265358979323846
#endif
#endif

#define EPOCH     2451543.5      /* Julian date of Dec 31 1999 0:00:00 UT */
#define AU        149597870.691  /* Astronomical Unit in km               */  
#define EARTHRAD  6378.163       /* Radius of Earth in km                 */
#define MANGLE    1873.7         /* Moon's angular size in arc seconds
				    at distance 60 Earth radii            */
#define SANGLE    1919.26        /* Sun's angular size in arc seconds
				    at distance 1 AU                      */


#define deg2rad(x) ((x) * PI/180.0)
#define rad2deg(x) ((x) * 180.0/PI)
#define fixangle(x) ((x) - 360.0 * (floor((x)/360.0)))       /* Fix angle */

#define sind(x) sin(deg2rad((x)))
#define cosd(x) cos(deg2rad((x)))


/* Orbital elements */
struct orbit {
  double N;     /* Longitude of the ascending node */
  double i;     /* Inclination to the ecliptic     */
  double w;     /* Argument of perihelion          */
  double a;     /* Semi-major axis                 */
  double e;     /* Numeric eccentricity            */
  double M;     /* Mean anomaly                    */
};

/* The sun and the moon */
static struct orbit s, m;


/*-------------------------------------------------------------------------*/
/* Function `eccentric_anomaly'                                            */
/* Solve Kepler's equation with Newton's method.                           */
/*-------------------------------------------------------------------------*/
double eccentric_anomaly(M, e)
     double M, e;
{
  double E = M, delta, eps = 1E-6;
  
  /* M is in radian. */
  do {
    delta = E - e * sin(E) - M ;
    E -= delta/(1 - e * cos(E));
  } while (abs(delta) >= eps);
 
  return(E);
}


/*-------------------------------------------------------------------------*/
/* Function `position_sun'                                                 */
/* The sun's distance and ecliptic longitude.                              */
/*-------------------------------------------------------------------------*/
double position_sun(d, lambda)
     double d, *lambda;
{
  double E, v, r, x, y;

  /* Orbital elements of the Sun */
  s.N = 0.0;
  s.i = 0.0;
  s.w = 282.9404 + 4.70935E-5 * d;
  s.a = 1.0;
  s.e = 0.016709 - 1.151E-9 * d;
  s.M = 356.0470 + 0.9856002585 * d;

  /* Eccentric anomaly (in radian).
   * Note: The eccentric anomaly E is related to the mean anomaly M
   * by Kepler's equation
   *   
   *   E = M + e sin(E)
   *
   * where e is the numeric eccentricity. If e is less about 0.05-0.06 
   * one may use the approximation
   *
   *   E = M + e * sin(M) * (1 + e * cos(M))
   */
  E = eccentric_anomaly(deg2rad(s.M), s.e);
  
  /* Get the true anomaly v and the distance r from eccentricity e 
   * and eccentric anomaly E. These are related by
   *
   *   r cos(v) = cos(E) - e
   *   r sin(v) = sqrt(1 - e*e) sin(E)
   *
   */
  x = cos(E) - s.e;
  y = sin(E) * sqrt(1.0 - s.e * s.e);
  
  v = atan2(y,x);
  r = sqrt(x * x + y * y);
  
  /* Sun's longitude in degrees. */
  *lambda = fixangle(rad2deg(v) + s.w);  
  
  return(r);
}


/*-------------------------------------------------------------------------*/
/* Function `position_moon'                                                */
/* The moon's distance, longitude and latitude.                            */
/*-------------------------------------------------------------------------*/
double position_moon(d, lambda, beta)
     double d, *lambda, *beta;
{
  double E, xi, eta, v, r, x, y, z;
  double Ls, Lm, D, F; 

  /* Orbital elements of the Moon. */
  m.N = 125.1228 - 0.0529538083 * d;
  m.i = 5.1454;
  m.w = 318.0634 + 0.1643573223 * d;
  m.a = 60.2666;                          /* Semi-major axis in Earth radii */
  m.e = 0.054900;
  m.M = 115.3654 + 13.0649929509 * d;

  /* Eccentric anomaly (in radian). */
  E = eccentric_anomaly(deg2rad(m.M), m.e);
  
  /* Get the true anomaly v and the distance r from eccentricity e 
   * and eccentric anomaly E. These are related by
   *
   *   r cos(v) = a (cos(E) - e)
   *   r sin(v) = a (sqrt(1 - e*e) sin(E))
   *
   */
  xi  = m.a * (cos(E) - m.e);
  eta = m.a * (sin(E) * sqrt(1.0 - m.e * m.e));
  
  v = rad2deg(atan2(eta,xi));
  r = sqrt(xi * xi + eta * eta);
 
  /* Get the position in space. */
  x = r * (cosd(m.N) * cosd(v + m.w) - sind(m.N) * sind(v + m.w) * cosd(m.i));
  y = r * (sind(m.N) * cosd(v + m.w) + cosd(m.N) * sind(v + m.w) * cosd(m.i));
  z = r * (sind(v + m.w) * sind(m.i));
  
  /* The moon's angular coordinates (in degrees). */
  *lambda = fixangle(rad2deg(atan2(y,x)));  
  *beta   = rad2deg(atan2(z,sqrt(x * x + y * y)));


  /* 
   * Now the most important perturbations of the moon's position 
   * are taken into account to achieve higher accuracy.
   */

  /* Mean longitude of the sun and the moon. */
  Ls = s.M + s.w;
  Lm = m.M + m.w + m.N;

  /* Mean elongation of the moon. */
  D = Lm - Ls;
  
  /* Argument of latitude for the moon. */
  F = Lm - m.N;

  /* Corrections of the ecliptic longitude. */
  *lambda += -1.274 * sind(m.M - 2*D)              /* Evection             */
    + 0.658 * sind(2*D)                            /* Variation            */
    - 0.186 * sind(s.M)                            /* Yearly Equation      */
    - 0.059 * sind(2*m.M - 2*D)
    - 0.057 * sind(m.M - 2*D + s.M)
    + 0.053 * sind(m.M + 2*D)
    + 0.046 * sind(2*D - s.M)
    + 0.041 * sind(m.M - s.M)
    - 0.035 * sind(D)                              /* Parallactic Equation */
    - 0.031 * sind(m.M + s.M)
    - 0.015 * sind(2*F - 2*D)
    + 0.011 * sind(m.M - 4*D);
  
  /* Corrections of the ecliptic latitude. */
  *beta += -0.173 * sind(F - 2*D)
    - 0.055 * sind(m.M - F - 2*D)
    - 0.046 * sind(m.M + F - 2*D)
    + 0.033 * sind(F + 2*D)
    + 0.017 * sind(2*m.M + F);
  
  /* Corrections of the distance. */
  r += -0.58 * cosd(m.M - 2*D) - 0.46 * cosd(2*D);

  *lambda = fixangle(*lambda); 
 
  return(r);
}


/*-------------------------------------------------------------------------*/
/* Function `phase2'                                                       */
/* Replacement for xmoontool's phase()                                     */
/*-------------------------------------------------------------------------*/
double phase2(jd, phase, mage, r_m, mangle, r_s, sangle)
  double  jd;
  double  *phase;                    /* Illuminated fraction */
  double  *mage;                     /* Age of moon in days */
  double  *r_m;                      /* Moon's distance in km */
  double  *mangle;                   /* Moon's angular diameter in degrees */
  double  *r_s;                      /* Sun's distance in km */
  double  *sangle;                   /* Sun's angular diameter in degrees */
{
  double d = jd - EPOCH, cos_elong, lambda_s, lambda_m, beta_m;
  

  /* The Sun */
  *r_s = position_sun(d, &lambda_s);
  *sangle = (SANGLE / *r_s) / 3600;
  *r_s *= AU;


  /* The Moon */
  *r_m = position_moon(d,&lambda_m,&beta_m);
  *mangle = (MANGLE * 60 / *r_m) / 3600;
  *r_m *= EARTHRAD;

  cos_elong = cosd(lambda_s - lambda_m) * cosd(beta_m);
  *phase = (1 - cos_elong) / 2;

  /* mage is no longer used. */
  *mage = 0;

  return(fixangle(lambda_m - lambda_s) / 360.0);
}
